/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999, 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Ant" and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.tools.ant.gui.core;

/** A Properties management class which knows how to load and store Properties.
 * Based on com.SoftWoehr.util.Properties from the
 * <a href="http://www.softwoehr.com">SoftWoehr Class Libraries</a>.
 * Add your default properties with add_default_property (String key, String property);
 * @author <a href="mailto:jax@softwoehr.com">Jack J. Woehr</a>
 * @version $Revision: 1.1 $
 */
public class PropertiesManager extends java.util.Properties {

  /** Represents the default properties set. Use default_properties.addProperty() to add
   * a default property.
   */
  private static java.util.Properties default_properties;

  /** Set or add a default property
   * @param key Name of property.
   * @param property Value of property
   */
  protected static void add_default_property (String key, String property) {
    default_properties.setProperty(key, property);
  }

  /** To respond to requests */
  public class NoSuchPropertyException extends java.lang.Exception {
    NoSuchPropertyException(String property) {
      super("Unknown property: " + property);
    }
  }

  /** To respond to requests */
  public class InvalidIntPropertyException extends java.lang.Exception {
    InvalidIntPropertyException(String property) {
      super("Invalid integer property: " + property);
    }
  }

  /** Create the defaults
   * Add any default properties you want to be hiding behind customized properties
   * to this static ctor using the set.Property method as illustrated below.
   */
  static {
    default_properties = new java.util.Properties ();

    /* Set all the defaults */
    default_properties.setProperty("org.apache.copyright", "Copyright (c) 1999, 2003 The Apache Software Foundation. All rights reserved");
    /* NOTE that these defaults are *not* written to the persistent properties file. D'oh, they're defaults! */
  }

  /** Creates new PropertiesManager possessing the default Properties */
  public PropertiesManager() {
    super(default_properties);
  }

  /** Load properties at creation
   * @param filename Properties file on local file system.
   */
  public PropertiesManager(String filename)
  throws java.io.FileNotFoundException, java.io.IOException
  {
    this();
    load_from_file(filename);
  }

  /** Reload saved properties from a local file
   * @param filename Local file to fetch property strings from
   * @throws FileNotFoundException Couldn't find that path.
   * @throws IOException Couldn't read for some reason.
   */
  public void load_from_file (String filename)
  throws java.io.FileNotFoundException, java.io.IOException
  {
    java.io.FileInputStream fi = new java.io.FileInputStream(filename);
    load(fi);
    fi.close();
  }

  /** Return a header suitable for a props file.
   * @return Text for header of a written properties file.
   */
  protected String get_properties_header () {
    return "Ant Antidote Properties loaded at Antidote start. This file is generated and modified automatically, but it may be hand-edited (carefully!)";
  }

  /** Save the properties
   * @param filename File on local file system to store props. to.
   * @throws FileNotFoundException Couldn't find that path.
   * @throws IOException Couldn't write for some reason.
   */
  public void store_to_file (String filename)
  throws java.io.FileNotFoundException, java.io.IOException
  {
    java.io.FileOutputStream fo = new java.io.FileOutputStream(filename);
    store(fo, get_properties_header ());
  }

  /** Get a specific int property.
   * @param property Name of property.
   * @throws NoSuchPropertyException No such.
   * @throws InvalidIntPropertyException Wasn't a base10 int.
   * @return Int value base 10 of property value
   */
  public int get_int_property (String property)
  throws NoSuchPropertyException, InvalidIntPropertyException
  {
    int value = 0;
    String prop = getProperty(property);
    if (null == prop)
    {
      throw new NoSuchPropertyException(property);
    }
    try {
      value = Integer.parseInt(prop);
    }
    catch (NumberFormatException e) {
      throw new InvalidIntPropertyException(property);
    }
    return value;
  }
}

